package come.stormborntales.notevault.ui.screens

import android.util.Log
import androidx.activity.compose.BackHandler
import androidx.compose.foundation.clickable
import androidx.compose.foundation.layout.Box
import androidx.compose.foundation.layout.Column
import androidx.compose.foundation.layout.Row
import androidx.compose.foundation.layout.Spacer
import androidx.compose.foundation.layout.fillMaxSize
import androidx.compose.foundation.layout.fillMaxWidth
import androidx.compose.foundation.layout.height
import androidx.compose.foundation.layout.padding
import androidx.compose.foundation.layout.wrapContentWidth
import androidx.compose.foundation.lazy.LazyColumn
import androidx.compose.foundation.lazy.items
import androidx.compose.material.icons.Icons
import androidx.compose.material.icons.filled.Add
import androidx.compose.material3.AlertDialog
import androidx.compose.material3.DropdownMenu
import androidx.compose.material3.DropdownMenuItem
import androidx.compose.material3.FloatingActionButton
import androidx.compose.material3.Icon
import androidx.compose.material3.ListItem
import androidx.compose.material3.MaterialTheme
import androidx.compose.material3.OutlinedTextField
import androidx.compose.material3.Scaffold
import androidx.compose.material3.Text
import androidx.compose.material3.TextButton
import androidx.compose.runtime.Composable
import androidx.compose.runtime.collectAsState
import androidx.compose.runtime.getValue
import androidx.compose.runtime.mutableStateOf
import androidx.compose.runtime.remember
import androidx.compose.runtime.setValue
import androidx.compose.ui.Alignment
import androidx.compose.ui.Modifier
import androidx.compose.ui.unit.dp
import androidx.lifecycle.ViewModelStoreOwner
import androidx.lifecycle.viewmodel.compose.LocalViewModelStoreOwner
import androidx.lifecycle.viewmodel.compose.viewModel
import come.stormborntales.notevault.data.local.AppDatabase
import come.stormborntales.notevault.data.local.entity.NoteCollection
import come.stormborntales.notevault.data.repository.CollectionRepository
import come.stormborntales.notevault.data.repository.NoteRepository
import come.stormborntales.notevault.ui.viewmodel.NoteBrowserViewModel
import come.stormborntales.notevault.ui.viewmodel.NoteBrowserViewModelFactory
import come.stormborntales.notevault.ui.viewmodel.NoteViewModelFactory

@Composable
fun CreateCollectionDialog(
    onDismiss: () -> Unit,
    onCreate: (String) -> Unit
) {
    var collectionName by remember { mutableStateOf("") }
    var isError by remember { mutableStateOf(false) }

    AlertDialog(
        onDismissRequest = onDismiss,
        title = {
            Text("Neue Collection erstellen")
        },
        text = {
            Column {
                OutlinedTextField(
                    value = collectionName,
                    onValueChange = {
                        collectionName = it
                        isError = false
                    },
                    label = { Text("Collection-Name") },
                    isError = isError,
                    singleLine = true
                )
                if (isError) {
                    Text(
                        text = "Name darf nicht leer sein",
                        color = MaterialTheme.colorScheme.error,
                        style = MaterialTheme.typography.bodySmall
                    )
                }
            }
        },
        confirmButton = {
            TextButton(onClick = {
                if (collectionName.isBlank()) {
                    isError = true
                } else {
                    onCreate(collectionName.trim())
                    onDismiss()
                }
            }) {
                Text("Erstellen")
            }
        },
        dismissButton = {
            TextButton(onClick = onDismiss) {
                Text("Abbrechen")
            }
        }
    )
}

@Composable
fun CollectionBreadcrumbs(
    path: List<NoteCollection>,
    onNavigateTo: (index: Int) -> Unit
) {
    Row(
        modifier = Modifier
            .fillMaxWidth()
            .padding(8.dp),
        verticalAlignment = Alignment.CenterVertically
    ) {
        BreadcrumbItem(label = "Start", onClick = { onNavigateTo(-1) })

        path.forEachIndexed { index, collection ->
            Text(" / ", style = MaterialTheme.typography.labelLarge)

            BreadcrumbItem(label = collection.name, onClick = {
                Log.d("Breadcrumb", "BreadcrumbItem " + collection.name + " was clicked")
                onNavigateTo(index)
            })
        }
    }
}

@Composable
private fun BreadcrumbItem(label: String, onClick: () -> Unit) {
    Box(
        modifier = Modifier
            .padding(horizontal = 4.dp)
            .clickable(onClick = {
                Log.d("Individual Breadcrumb", "A simple test " + label)
                onClick()
            })
    ) {
        Text(
            text = label,
            style = MaterialTheme.typography.labelLarge,
        )
    }
}

@Composable
fun NotesScreen(
    collectionRepository: CollectionRepository,
    noteRepository: NoteRepository,

) {
    var menuExpanded by remember { mutableStateOf(false) }
    var showDialog by remember { mutableStateOf(false) }
    val viewModelStoreOwner = checkNotNull(LocalViewModelStoreOwner.current)
    val viewModel: NoteBrowserViewModel = viewModel(
        viewModelStoreOwner,
        factory = NoteBrowserViewModelFactory(noteRepository, collectionRepository)
    )

    val collections by viewModel.currentCollections.collectAsState()
    val notes by viewModel.currentNotes.collectAsState()

    Scaffold(
        floatingActionButton = {
            Box {
                FloatingActionButton(onClick = { menuExpanded = true }) {
                    Icon(Icons.Default.Add, contentDescription = "Open menu")
                }

                DropdownMenu(
                    expanded = menuExpanded,
                    onDismissRequest = { menuExpanded = false },
                    modifier = Modifier
                        .wrapContentWidth()
                ) {
                    DropdownMenuItem(
                        text = { Text("Ordner erstellen") },
                        onClick = {
                            menuExpanded = false
                            showDialog = true
                        }
                    )
                    DropdownMenuItem(
                        text = { Text("Noten importieren") },
                        onClick = {
                            menuExpanded = false
                            // Handle action
                        }
                    )
                }
            }
        },
        content = { innerPadding ->

            val path by viewModel.pathStack.collectAsState()

            BackHandler(enabled = path.isNotEmpty()) {
                viewModel.navigateUp()
            }

            Column(
                modifier = Modifier
                    .fillMaxSize()
                    .padding(innerPadding)
            ) {
                // Breadcrumbs sichtbar & klickbar machen
                CollectionBreadcrumbs(
                    path = path,
                    onNavigateTo = { index ->
                        Log.d("NoteBrowser", "Navigate to: $index")
                        if (index == -1) viewModel.loadContentForParent(null)
                        else viewModel.navigateToLevel(index)
                    }
                )

                LazyColumn {
                    item {
                        Text(
                            "Ordner",
                            style = MaterialTheme.typography.titleMedium,
                            modifier = Modifier.padding(16.dp)
                        )
                    }

                    items(collections) { collection ->
                        ListItem(
                            headlineContent = { Text(collection.name) },
                            modifier = Modifier
                                .clickable {
                                    viewModel.loadContentForParent(collection.id, collection)
                                }
                                .padding(horizontal = 8.dp)
                        )
                    }

                    item {
                        Spacer(modifier = Modifier.height(16.dp))
                        Text(
                            "Noten",
                            style = MaterialTheme.typography.titleMedium,
                            modifier = Modifier.padding(16.dp)
                        )
                    }

                    items(notes) { note ->
                        ListItem(
                            headlineContent = { Text(note.title) },
                            supportingContent = {
                                note.composer?.let { Text(it) }
                            },
                            modifier = Modifier.padding(horizontal = 8.dp)
                        )
                    }
                }
            }



            if (showDialog) {
                CreateCollectionDialog(
                    onDismiss = { showDialog = false },
                    onCreate = { name ->
                        // Handle the new collection name
                        viewModel.createCollection(name)
                    }
                )
            }
        }
    )
}
